@@ -7,59 +7,30 @@ import sys
 import threading
 import time
 from types import FunctionType, MethodType, ModuleType
-from typing import Any, List, Tuple, Union
+from typing import Any, Dict, Type, Union
 from uuid import UUID
 
-from arrlio import __tasks__, settings
+from arrlio import __tasks__
 from arrlio.exc import NotFoundError, TaskError, TaskNoResultError, TaskTimeoutError
 from arrlio.models import Event, Graph, Message, Task, TaskData, TaskInstance, TaskResult
 from arrlio.settings import Config
+from arrlio.tp import AsyncCallableT
 
 
 logger = logging.getLogger("arrlio")
 
 
-def task(
-    func: FunctionType = None,
-    name: str = None,
-    bind: bool = None,
-    base: Task = None,
-    queue: str = None,
-    priority: int = None,
-    timeout: int = None,
-    ttl: int = None,
-    ack_late: bool = None,
-    result_ttl: int = None,
-    result_return: bool = None,
-    result_encrypt: bool = None,
-    thread: bool = None,
-    events: bool = None,
-    event_ttl: int = None,
-) -> Task:
+def task(func: FunctionType = None, name: str = None, base: Type[Task] = None, **kwds):
+    """
+    Args:
+        func (FunctionType, optional): Task function.
+        name (str, optional): ~arrlio.models.Task name.
+        base (~arrlio.models.Task, optional): ~arrlio.models.Task base class.
+        kwds (dict, optional): ~arrlio.models.TaskData arguments.
+    """
 
-    if bind is None:
-        bind = settings.TASK_BIND
     if base is None:
         base = Task
-    if queue is None:
-        queue = settings.TASK_QUEUE
-    if priority is None:
-        priority = settings.TASK_PRIORITY
-    if timeout is None:
-        timeout = settings.TASK_TIMEOUT
-    if ttl is None:
-        ttl = settings.TASK_TTL
-    if ack_late is None:
-        ack_late = settings.TASK_ACK_LATE
-    if result_ttl is None:
-        result_ttl = settings.TASK_RESULT_TTL
-    if result_return is None:
-        result_return = settings.TASK_RESULT_RETURN
-    if events is None:
-        events = settings.EVENTS
-    if event_ttl is None:
-        event_ttl = settings.EVENT_TTL
-
     if func is not None:
         if not isinstance(func, (FunctionType, MethodType)):
             raise TypeError("Argument 'func' does not a function or method")
@@ -67,45 +38,14 @@ def task(
             name = f"{func.__module__}.{func.__name__}"
         if name in __tasks__:
             raise ValueError(f"Task '{name}' already registered")
-        t = base(
-            func=func,
-            name=name,
-            bind=bind,
-            queue=queue,
-            priority=priority,
-            timeout=timeout,
-            ttl=ttl,
-            ack_late=ack_late,
-            result_ttl=result_ttl,
-            result_return=result_return,
-            result_encrypt=result_encrypt,
-            thread=thread,
-            events=events,
-            event_ttl=event_ttl,
-        )
+        t = base(func=func, name=name, **kwds)
         __tasks__[name] = t
         logger.info("Register %s", t)
         return t
     else:
 
         def wrapper(func):
-            return task(
-                func=func,
-                name=name,
-                bind=bind,
-                base=base,
-                queue=queue,
-                priority=priority,
-                timeout=timeout,
-                ttl=ttl,
-                ack_late=ack_late,
-                result_ttl=result_ttl,
-                result_return=result_return,
-                result_encrypt=result_encrypt,
-                thread=thread,
-                events=events,
-                event_ttl=event_ttl,
-            )
+            return task(base=base, func=func, name=name, **kwds)
 
         return wrapper
 
@@ -118,8 +58,8 @@ class Executor:
         return self.__str__()
 
     async def __call__(self, task_instance: TaskInstance) -> TaskResult:
-        task_data: TaskData = task_instance.data
         task: Task = task_instance.task
+        task_data: TaskData = task_instance.data
 
         res, exc, trb = None, None, None
         t0 = time.monotonic()
@@ -127,23 +67,24 @@ class Executor:
         logger.info("%s: execute task %s(%s)", self, task.name, task_data.task_id)
 
         try:
-            if task_instance.task.func is None:
-                raise NotFoundError(f"Task '{task_instance.task.name}' not found")
 
-            kwdefaults = task_instance.task.func.__kwdefaults__
-            meta = kwdefaults and "meta" in kwdefaults
+            if task.func is None:
+                raise NotFoundError(f"Task '{task.name}' not found")
+
+            kwdefaults = task.func.__kwdefaults__
+            meta: bool = kwdefaults is not None and "meta" in kwdefaults
 
             try:
-                if inspect.iscoroutinefunction(task_instance.task.func):
+                if inspect.iscoroutinefunction(task.func):
                     res = await asyncio.wait_for(task_instance(meta=meta), task_data.timeout)
                 else:
                     res = task_instance(meta=meta)
             except asyncio.TimeoutError:
                 raise TaskTimeoutError(task_data.timeout)
+
         except Exception as e:
             exc_info = sys.exc_info()
-            exc = exc_info[1]
-            trb = exc_info[2]
+            exc, trb = exc_info[1], exc_info[2]
             if isinstance(e, TaskTimeoutError):
                 logger.error("Task timeout for %s", task_instance)
             else:
@@ -190,22 +131,20 @@ class ThreadExecutor(Executor):
 
 
 class App:
-    def __init__(
-        self,
-        config: Config,
-        # backend_config_kwds: dict = None,
-    ):
+    """
+    Args:
+        config (~arrlio.settings.Config): Arrlio application config.
+    """
+
+    def __init__(self, config: Config):
         self.config = config
         if isinstance(config.backend, ModuleType):
-            self._backend = self.config.backend.Backend(
-                # self.config.backend.BackendConfig(**(backend_config_kwds or {}))
-                self.config.backend.BackendConfig()
-            )
+            self._backend = self.config.backend.Backend(self.config.backend.BackendConfig())
         else:
             self._backend = self.config.backend()
         self._closed: asyncio.Future = asyncio.Future()
-        self._running_tasks: dict = {}
-        self._running_messages: dict = {}
+        self._running_tasks: Dict[UUID, asyncio.Task] = {}
+        self._running_messages: Dict[UUID, asyncio.Task] = {}
         self._lock: asyncio.Lock = asyncio.Lock()
 
     def __str__(self):
@@ -225,7 +164,7 @@ class App:
         return self._backend
 
     @property
-    def is_closed(self):
+    def is_closed(self) -> bool:
         return self._closed.done()
 
     async def close(self):
@@ -235,26 +174,26 @@ class App:
         await self._backend.close()
         self._closed.set_result(None)
 
-    async def run_task(
+    async def send_task(
         self,
         task: Union[Task, str],
         args: tuple = None,
         kwds: dict = None,
-        queue: str = None,
-        priority: int = None,
-        timeout: int = None,
-        ttl: int = None,
-        encrypt: bool = None,
-        ack_late: bool = None,
-        result_ttl: int = None,
-        result_return: bool = None,
-        result_encrypt: bool = None,
-        thread: bool = None,
-        events: bool = None,
-        event_ttl: int = None,
-        extra: dict = None,
+        backend_extra: dict = None,
         **kwargs,
     ) -> "AsyncResult":
+        """
+        Args:
+            task (Union[~arrlio.models.Task, str]): ~arrlio.models.Task of task name.
+            args (tuple, optional): Task args.
+            kwds (dict, optional): Task kwds.
+            backend_extra (dict, optional): ~arrlio.models.TaskData backend_extra.
+            kwargs (dict, optional): Other ~arrlio.models.TaskData arguments.
+
+        Returns:
+            AsyncResult: Task ~arrlio.core.AsyncResult.
+        """
+
         name = task
         if isinstance(task, Task):
             name = task.name
@@ -263,119 +202,94 @@ class App:
             args = ()
         if kwds is None:
             kwds = {}
-        if extra is None:
-            extra = {}
+        if backend_extra is None:
+            backend_extra = {}
 
-        settings = self.config.task.dict(exclude_unset=True)
-
-        if queue is None:
-            queue = settings.get("queue")
-        if priority is None:
-            priority = settings.get("priority")
-        if timeout is None:
-            timeout = settings.get("timeout")
-        if ttl is None:
-            ttl = settings.get("ttl")
-        if ack_late is None:
-            ack_late = settings.get("ack_late")
-        if result_ttl is None:
-            result_ttl = settings.get("result_ttl")
-        if result_return is None:
-            result_return = settings.get("result_return")
-        if events is None:
-            events = settings.get("events")
-        if event_ttl is None:
-            event_ttl = settings.get("event_ttl")
-
-        task_data = TaskData(
-            args=args,
-            kwds=kwds,
-            queue=queue,
-            priority=priority,
-            timeout=timeout,
-            ttl=ttl,
-            encrypt=encrypt,
-            ack_late=ack_late,
-            result_ttl=result_ttl,
-            result_return=result_return,
-            result_encrypt=result_encrypt,
-            thread=thread,
-            events=events,
-            event_ttl=event_ttl,
-            extra=extra,
-        )
+        task_settings = self.config.task.dict(exclude_unset=True)
 
         if name in __tasks__:
-            task_instance = __tasks__[name].instantiate(data=task_data)
+            task_instance = __tasks__[name].instantiate(
+                args=args, kwds=kwds, backend_extra=backend_extra, **{**task_settings, **kwargs}
+            )
         else:
-            task_instance = Task(None, name).instantiate(data=task_data)
+            task_instance = Task(None, name).instantiate(
+                args=args, kwds=kwds, backend_extra=backend_extra, **{**task_settings, **kwargs}
+            )
 
-        logger.info("%s: run %s", self, task_instance)
+        logger.info("%s: send %s", self, task_instance)
 
-        await self._backend.send_task(task_instance, **kwargs)
+        await self._backend.send_task(task_instance)
 
         return AsyncResult(self, task_instance)
 
-    async def run_graph(self, graph: Graph, args: Union[Tuple, List] = None, kwds: dict = None, meta: dict = None):
+    async def send_graph(
+        self,
+        graph: Graph,
+        args: tuple = None,
+        kwds: dict = None,
+        meta: dict = None,
+    ) -> Dict[str, "AsyncResult"]:
+        """
+        Args:
+            graph (Graph): ~arrlio.models.Graph.
+            args (tuple, optional): ~arrlio.models.Graph root nodes args.
+            kwds (dict, optional): ~arrlio.models.Graph root nodes kwds.
+            meta (dict, optional): ~arrlio.models.Graph root nodes meta.
+
+        Returns:
+            Dict[str, ~arrlio.core.AsyncResult]: Dictionary with AsyncResult objects.
+        """
+
         nodes = copy.deepcopy(graph.nodes)
         edges = graph.edges
         roots = graph.roots
 
         if not nodes or not roots:
-            logger.warning("Empty graph or missing roots")
-            return
+            raise ValueError("Empty graph or missing roots")
 
-        logger.info("%s: run %s with args: %s and kwds: %s", self, graph, args, kwds)
+        logger.info("%s: send %s with args: %s and kwds: %s", self, graph, args, kwds)
 
         task_instances = {}
         for node_name, (task, node_kwds) in nodes.items():
             if node_name not in roots and node_kwds.get("task_id"):
                 continue
-            task_data = TaskData(**node_kwds)
-            node_kwds["task_id"] = task_data.task_id
             if task in __tasks__:
-                task_instance = __tasks__[task].instantiate(data=task_data)
+                task_instance = __tasks__[task].instantiate(**node_kwds)
             else:
-                task_instance = Task(None, task).instantiate(data=task_data)
+                task_instance = Task(None, task).instantiate(**node_kwds)
+            node_kwds["task_id"] = task_instance.data.task_id
             task_instances[node_name] = task_instance
 
         for root in roots:
-            task_instances[root].data.args += tuple(args or ())
-            task_instances[root].data.kwds.update(kwds or {})
-            task_instances[root].data.meta.update(meta or {})
-            task_instances[root].data.graph = Graph(graph.id, nodes=nodes, edges=edges, roots={root})
-            logger.info("%s: run %s", self, task_instances[root])
+            data = task_instances[root].data
+            data.args += tuple(args or ())
+            data.kwds.update(kwds or {})
+            data.meta.update(meta or {})
+            data.graph = Graph(graph.id, nodes=nodes, edges=edges, roots={root})
+
+            logger.info("%s: send %s", self, task_instances[root])
+
             await self._backend.send_task(task_instances[root])
 
         return {k: AsyncResult(self, task_instance) for k, task_instance in task_instances.items()}
 
-    async def send_message(
-        self,
-        message: Any,
-        exchange: str = None,
-        routing_key: str = None,
-        priority: int = None,
-        ttl: int = None,
-        ack_late: bool = None,
-        encrypt: bool = None,
-    ):
-        settings = self.config.message.dict(exclude_unset=True)
+    async def send_message(self, data: Any, routing_key: str = None, **kwds):
+        """
+        Args:
+            data (Any): Message data.
+            routing_key (str, optional): Message routing key.
+            kwds (dict, optional): ~arrlio.models.Message arguments.
+        """
 
-        if exchange is None:
-            exchange = settings.get("exchange")
-        if priority is None:
-            priority = settings.get("priority")
-        if ttl is None:
-            ttl = settings.get("ttl")
-        if ack_late is None:
-            ack_late = settings.get("ack_late")
+        message_settings = self.config.message.dict(exclude_unset=True)
+        message = Message(data=data, **{**message_settings, **kwds})
 
-        message = Message(exchange=exchange, data=message, priority=priority, ttl=ttl, ack_late=ack_late)
         logger.info("%s: send %s", self, message)
-        await self._backend.send_message(message, routing_key=routing_key, encrypt=encrypt)
+
+        await self._backend.send_message(message, routing_key=routing_key)
 
     async def pop_result(self, task_instance: TaskInstance):
-        task_result = await self._backend.pop_task_result(task_instance)
+        task_result: TaskResult = await self._backend.pop_task_result(task_instance)
         if task_result.exc:
             if isinstance(task_result.exc, TaskError):
                 raise task_result.exc
@@ -385,8 +299,8 @@ class App:
 
     async def consume_tasks(self):
         if self.config.task_queues:
-            logger.info("%s: consuming task queues %s", self, self.config.task_queues)
             await self._backend.consume_tasks(self.config.task_queues, self._on_task)
+            logger.info("%s: consuming task queues %s", self, self.config.task_queues)
 
     async def stop_consume_tasks(self):
         async with self._lock:
@@ -395,11 +309,55 @@ class App:
                 logger.debug("Cancel processing task '%s'", task_id)
                 aio_task.cancel()
             self._running_tasks = {}
+            logger.info("%s: stop consuming task queues %s", self, self.config.task_queues)
+
+    def make_event(self, event_type: str, event_data: dict = None) -> Event:
+        return Event(
+            type=event_type,
+            dt=datetime.datetime.now(tz=datetime.timezone.utc),
+            data=event_data or {},
+        )
+
+    async def on_task_received(self, task_instance: TaskInstance):
+        task_type = "task received"
+        task_data = task_instance.data
+        if (
+            task_data.events is True
+            or isinstance(task_data.events, (list, set, tuple))
+            and task_type in task_data.events
+        ):
+            event: Event = self.make_event(
+                task_type,
+                event_data={"task_id": task_instance.data.task_id},
+            )
+            await self._backend.push_event(event)
+
+    async def on_task_done(self, task_instance: TaskInstance, task_result: TaskResult):
+        task_type = "task done"
+        task_data = task_instance.data
+        if (
+            task_data.events is True
+            or isinstance(task_data.events, (list, set, tuple))
+            and task_type in task_data.events
+        ):
+            event: Event = self.make_event(
+                task_type,
+                event_data={"task_id": task_instance.data.task_id, "status": task_result.exc is None},
+            )
+            await self._backend.push_event(event)
 
     async def _on_task(self, task_instance: TaskInstance):
         try:
+            await self.on_task_received(task_instance)
+
             task_id: UUID = task_instance.data.task_id
 
+            def fn():
+                aio_task = asyncio.create_task(self.execute_task(task_instance))
+                aio_task.add_done_callback(lambda *args: self._running_tasks.pop(task_id, None))
+                self._running_tasks[task_id] = aio_task
+                return aio_task
+
             async with self._lock:
                 if self.is_closed:
                     return
@@ -407,23 +365,18 @@ class App:
                 if len(self._running_tasks) + len(self._running_messages) + 1 >= self.config.pool_size:
                     await self._backend.stop_consume_tasks()
                     try:
-                        aio_task = asyncio.create_task(self._execute_task(task_instance))
-                        aio_task.add_done_callback(lambda *args: self._running_tasks.pop(task_id, None))
-                        self._running_tasks[task_id] = aio_task
-                        await aio_task
+                        await fn()
                     finally:
                         if not self.is_closed:
                             await self._backend.consume_tasks(self.config.task_queues, self._on_task)
                     return
 
-            aio_task = asyncio.create_task(self._execute_task(task_instance))
-            aio_task.add_done_callback(lambda *args: self._running_tasks.pop(task_id, None))
-            self._running_tasks[task_id] = aio_task
+            fn()
 
         except Exception as e:
             logger.exception(e)
 
-    async def _execute_task(self, task_instance: TaskInstance):
+    async def execute_task(self, task_instance: TaskInstance):
         try:
             task_data: TaskData = task_instance.data
 
@@ -446,14 +399,22 @@ class App:
                     for node_id, node_id_routes in graph.edges[root]:
                         if not ((routes is None and node_id_routes is None) or set(routes) & set(node_id_routes)):
                             continue
-                        graph: Graph = Graph(
-                            id=graph.id,
-                            nodes=graph.nodes,
-                            edges=graph.edges,
-                            roots={node_id},
+                        # graph: Graph = Graph(
+                        #     id=graph.id,
+                        #     nodes=graph.nodes,
+                        #     edges=graph.edges,
+                        #     roots={node_id},
+                        # )
+                        await self.send_graph(
+                            Graph(
+                                id=graph.id,
+                                nodes=graph.nodes,
+                                edges=graph.edges,
+                                roots={node_id},
+                            ),
+                            args=args,
+                            meta={"source_node": root},
                         )
-                        logger.error(args)
-                        await self.run_graph(graph, args=args, meta={"source_node": root})
 
             if task_instance.data.result_return:
                 try:
@@ -461,26 +422,14 @@ class App:
                 except Exception as e:
                     logger.exception(e)
 
-            if task_instance.data.events:
-                event: Event = Event(
-                    type="task done",
-                    datetime=datetime.datetime.now(tz=datetime.timezone.utc),
-                    data={
-                        "task_id": task_instance.data.task_id,
-                        "status": task_result.exc is None,
-                    },
-                )
-                try:
-                    await self._backend.push_event(task_instance, event)
-                except Exception as e:
-                    logger.exception(e)
+            await self.on_task_done(task_instance, task_result)
 
         except asyncio.CancelledError:
             pass
         except Exception as e:
             logger.exception(e)
 
-    async def consume_messages(self, on_message):
+    async def consume_messages(self, on_message: AsyncCallableT):
         if self.config.message_queues:
 
             async def handle(message: Message):
@@ -518,7 +467,7 @@ class App:
                 aio_task.cancel()
             self._running_messages = {}
 
-    async def consume_events(self, on_event):
+    async def consume_events(self, on_event: AsyncCallableT):
         logger.info("%s: consuming events", self)
         await self._backend.consume_events(on_event)
 
